package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.EvaluationFormDTO;
import digital.laboratory.platform.reagent.entity.EvaluationForm;
import digital.laboratory.platform.reagent.entity.ProvideServicesOrSupplies;
import digital.laboratory.platform.reagent.entity.PurchaseListDetails;
import digital.laboratory.platform.reagent.entity.SupplierInformation;
import digital.laboratory.platform.reagent.mapper.EvaluationFormMapper;
import digital.laboratory.platform.reagent.service.EvaluationFormService;
import digital.laboratory.platform.reagent.service.ProvideServicesOrSuppliesService;
import digital.laboratory.platform.reagent.service.PurchaseListDetailsService;
import digital.laboratory.platform.reagent.service.SupplierInformationService;
import digital.laboratory.platform.reagent.vo.EvaluationFormVO;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * (服务商/供应商评价表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (服务商 / 供应商评价表) 服务实现类
 */
@SuppressWarnings("all")
@Service
@RequiredArgsConstructor
public class EvaluationFormServiceImpl extends ServiceImpl<EvaluationFormMapper, EvaluationForm> implements EvaluationFormService {


    private EvaluationFormService evaluationFormService;

    private ProvideServicesOrSuppliesService provideServicesOrSuppliesService;


    private SupplierInformationService supplierInformationService;


    private PurchaseListDetailsService purchaseListDetailsService;

    @Override//通过ID查询评价表
    public EvaluationForm getFormById(String evaluationFormId) {

        EvaluationFormVO evaluationForm = baseMapper.getEvaluationForm(evaluationFormId);

        evaluationForm.setProvideServicesOrSuppliesVOList(provideServicesOrSuppliesService.getProvideServicesOrSuppliesVOList(evaluationFormId));

        return evaluationForm;
    }

    @Override//增加/修改评价表
    public EvaluationForm addFormById(EvaluationFormDTO evaluationFormDTO,DLPUser dlpUser) {

        if (evaluationFormService.getById(evaluationFormDTO.getEvaluationFormId()) != null) {

            LambdaQueryWrapper<ProvideServicesOrSupplies> provideServicesOrSuppliesLambdaQueryWrapper = new LambdaQueryWrapper<>();

            provideServicesOrSuppliesLambdaQueryWrapper.eq(ProvideServicesOrSupplies::getEvaluationFormId, evaluationFormDTO.getEvaluationFormId());
            //先删除提供的供应品集合
            provideServicesOrSuppliesService.removeBatchByIds(provideServicesOrSuppliesService.list(provideServicesOrSuppliesLambdaQueryWrapper));

            EvaluationForm byId = evaluationFormService.getById(evaluationFormDTO.getEvaluationFormId());

            BeanUtils.copyProperties(evaluationFormDTO, byId);

            byId.setPrimaryUserId(dlpUser.getId());

            byId.setCommentsDateFromPrimary(LocalDate.now());

            byId.setStatus(1);

            List<ProvideServicesOrSupplies> list = new ArrayList<>();
            //再通过供应商ID，在采购清单列表里面，查找对应供应商提供的所有试剂耗材ID
            SupplierInformation byId1 = supplierInformationService.getById(evaluationFormDTO.getSupplierInformationId());

            LambdaQueryWrapper<PurchaseListDetails> purchaseListDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchaseListDetailsLambdaQueryWrapper.eq(PurchaseListDetails::getSupplierId, byId1.getSupplierInformationId());

            List<PurchaseListDetails> purchaseListDetailsList = purchaseListDetailsService.list(purchaseListDetailsLambdaQueryWrapper);

            for (PurchaseListDetails purchaseListDetails : purchaseListDetailsList) {

                ProvideServicesOrSupplies provideServicesOrSupplies = new ProvideServicesOrSupplies();

                provideServicesOrSupplies.setReagentConsumableId(purchaseListDetails.getReagentConsumableId());

                provideServicesOrSupplies.setEvaluationFormId(byId.getEvaluationFormId());

                provideServicesOrSupplies.setProvideServicesOrSuppliesId(IdWorker.get32UUID().toUpperCase());

                list.add(provideServicesOrSupplies);
            }
            if (evaluationFormService.updateById(byId) & provideServicesOrSuppliesService.saveBatch(list)) {
                return byId;
            } else {
                return null;
            }
        } else {

            EvaluationForm evaluationForm = new EvaluationForm();

            BeanUtils.copyProperties(evaluationFormDTO,evaluationForm);

            evaluationForm.setStatus(1);

            evaluationForm.setEvaluationFormId(IdWorker.get32UUID().toUpperCase());

            evaluationForm.setPrimaryUserId(dlpUser.getId());

            evaluationForm.setCommentsDateFromPrimary(LocalDate.now());

            List<ProvideServicesOrSupplies> list = new ArrayList<>();
            //再通过供应商ID，在采购清单列表里面，查找对应供应商提供的所有试剂耗材ID
            SupplierInformation byId1 = supplierInformationService.getById(evaluationFormDTO.getSupplierInformationId());

            LambdaQueryWrapper<PurchaseListDetails> purchaseListDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchaseListDetailsLambdaQueryWrapper.eq(PurchaseListDetails::getSupplierId, byId1.getSupplierInformationId());

            List<PurchaseListDetails> purchaseListDetailsList = purchaseListDetailsService.list(purchaseListDetailsLambdaQueryWrapper);

            for (PurchaseListDetails purchaseListDetails : purchaseListDetailsList) {

                ProvideServicesOrSupplies provideServicesOrSupplies = new ProvideServicesOrSupplies();

                provideServicesOrSupplies.setReagentConsumableId(purchaseListDetails.getReagentConsumableId());

                provideServicesOrSupplies.setEvaluationFormId(evaluationForm.getEvaluationFormId());

                provideServicesOrSupplies.setProvideServicesOrSuppliesId(IdWorker.get32UUID().toUpperCase());

                list.add(provideServicesOrSupplies);
            }

            if (evaluationFormService.save(evaluationForm)&provideServicesOrSuppliesService.saveBatch(list)){
                return evaluationForm;
            }else {
                return null;
            }

        }

    }

    @Override//二级审核
    public EvaluationForm auditFormOfSecondary(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser){

        EvaluationForm byId = evaluationFormService.getById(auditAndApproveDTO.getUuId());

        byId.setSecondaryUserId(dlpUser.getId());

        byId.setCommentsDateFromSecondary(LocalDate.now());

        byId.setCommentsFromSecondary(auditAndApproveDTO.getAuditOpinion());

        byId.setCommentsResultFromSecondary(auditAndApproveDTO.getAuditResult());

        if (evaluationFormService.updateById(byId)){
            return byId;
        }else{
            return null;
        }
    }

    @Override//三级审核

    public EvaluationForm auditFormOfThreeLevel(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser){

        EvaluationForm byId = evaluationFormService.getById(auditAndApproveDTO.getUuId());

        byId.setSecondaryUserId(dlpUser.getId());

        byId.setCommentsDateFromThreeLevel(LocalDate.now());

        byId.setCommentsFromThreeLevel(auditAndApproveDTO.getAuditOpinion());

        byId.setCommentsResultFromThreeLevel(auditAndApproveDTO.getAuditResult());

        byId.setStatus(2);

        if (evaluationFormService.updateById(byId)){
            return byId;
        }else{
            return null;
        }
    }
}









