package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.dto.PurchaseListDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.PurchaseListMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.PurchaseListVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (采购清单)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购清单) 服务实现类
 */
@Service
public class PurchaseListServiceImpl extends ServiceImpl<PurchaseListMapper, PurchaseList> implements PurchaseListService {

    @Autowired
    private PurchaseListService purchaseListService;
    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Autowired
    private SigningRecordFormService signingRecordFormService;

    @Autowired
    private CatalogueDetailsService catalogueDetailsService;

    @Autowired
    private SignedContentService signedContentService;

    @Override//通过ID查找清单
    public PurchaseListVO getPurchaseList(String purchaseListId) {

        PurchaseList byId = purchaseListService.getById(purchaseListId);

        List<PurchaseListDetails> list = purchaseListDetailsService.getPurchaseListDetailsVOList(purchaseListId);

        PurchaseListVO purchaseListVO = new PurchaseListVO();

        BeanUtils.copyProperties(byId, purchaseListVO);

        purchaseListVO.setPurchaseListDetailsList(list);

        return purchaseListVO;
    }

    @Transactional
    @Override//新增采购清单
    public PurchaseList addListById(List<PurchaseListDTO> purchaseListDTOList, PurchaseList purchaseList){

        purchaseList.setPurchaseListId(IdWorker.get32UUID().toUpperCase());

        purchaseList.setName(LocalDate.now()+"采购清单");

        List<PurchaseListDetails> purchaseListDetailsList= new ArrayList<>();
        //创建签收内容集合
        List<SignedContent> signedContentList = new ArrayList<>();
        //创建签收记录表
        SigningRecordForm signingRecordForm = new SigningRecordForm();
        //完善签收记录表信息
        signingRecordForm.setSigningRecordFormId(IdWorker.get32UUID().toUpperCase());

        signingRecordForm.setPurchaseListId(purchaseList.getPurchaseListId() );

        signingRecordForm.setPurchaseNumber(LocalDate.now()+"签收任务");

        for (PurchaseListDTO purchaseListDTO : purchaseListDTOList) {

            PurchaseListDetails purchaseListDetails = new PurchaseListDetails();

            BeanUtils.copyProperties(purchaseListDTO,purchaseListDetails);

            purchaseListDetails.setPurchaseListId(purchaseList.getPurchaseListId());

            purchaseListDetails.setPurchaselistDetailsId(IdWorker.get32UUID().toUpperCase());

            purchaseListDetailsList.add(purchaseListDetails);

            SignedContent signedContent = new SignedContent();
            //完善签收内容信息
            signedContent.setSignedContentId(IdWorker.get32UUID().toUpperCase());

            signedContent.setTotalQuantity(purchaseListDetails.getPurchaseQuantity());

            signedContent.setReagentConsumableId(purchaseListDetails.getReagentConsumableId());

            signedContent.setSigningRecordFormId(signingRecordForm.getSigningRecordFormId());

            LambdaQueryWrapper<CatalogueDetails> catalogueDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

            catalogueDetailsLambdaQueryWrapper.eq(CatalogueDetails::getReagentConsumableId,signedContent.getReagentConsumableId());

            List<CatalogueDetails> list = catalogueDetailsService.list(catalogueDetailsLambdaQueryWrapper);

            signedContent.setCatalogueNumber(list.get(0).getNumber());

            signedContentList.add(signedContent);

        }
        if (purchaseListService.save(purchaseList)&purchaseListDetailsService.saveBatch(purchaseListDetailsList)&signingRecordFormService.save(signingRecordForm)
                &signedContentService.saveBatch(signedContentList)){
            return purchaseList;
        }else {
            return null;
        }
    }
}
