package digital.laboratory.platform.reagent.service.impl;

import com.alibaba.nacos.shaded.org.checkerframework.checker.units.qual.A;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.SigningRecordFormDTO;
import digital.laboratory.platform.reagent.entity.SignedBatchList;
import digital.laboratory.platform.reagent.entity.SignedContent;
import digital.laboratory.platform.reagent.entity.SigningRecordForm;
import digital.laboratory.platform.reagent.entity.StorageRegistrationForm;
import digital.laboratory.platform.reagent.mapper.SigningRecordFormMapper;
import digital.laboratory.platform.reagent.service.SignedBatchListService;
import digital.laboratory.platform.reagent.service.SignedContentService;
import digital.laboratory.platform.reagent.service.SigningRecordFormService;
import digital.laboratory.platform.reagent.service.StorageRegistrationFormService;
import digital.laboratory.platform.reagent.vo.SignedBatchListVO;
import digital.laboratory.platform.reagent.vo.SignedContentVO;
import digital.laboratory.platform.reagent.vo.SigningRecordFormVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 签收记录表服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 签收记录表 服务实现类
 */
@Service
public class SigningRecordFormServiceImpl extends ServiceImpl<SigningRecordFormMapper, SigningRecordForm> implements SigningRecordFormService {
    @Autowired
    private SigningRecordFormService signingRecordFormService;

    @Autowired
    private SignedContentService signedContentService;

    @Autowired
    private SignedBatchListService signedBatchListService;

    @Autowired
    private StorageRegistrationFormService storageRegistrationFormService;
    ;

//    @Override
//    public IPage<SigningRecordFormVO> getSigningRecordFormVOPage(IPage<SigningRecordForm> page, QueryWrapper<SigningRecordForm> qw) {
//
//        IPage<SigningRecordFormVO> signingRecordFormVOPage = baseMapper.getSigningRecordFormVOPage(page, qw);
//
//        return signingRecordFormVOPage;
//    }

//    @Override
//    public List<SigningRecordFormVO> getSigningRecordFormVOList(QueryWrapper<SigningRecordForm> qw) {
//
//        List<SigningRecordFormVO> signingRecordFormVOList = baseMapper.getSigningRecordFormVOList(qw);
//
//        return signingRecordFormVOList;
//    }

    @Override
    public SigningRecordFormVO getSigningRecordFormVO(String signingRecordFormId) {

        SigningRecordForm byId = signingRecordFormService.getById(signingRecordFormId);

        SigningRecordFormVO signingRecordFormVO = new SigningRecordFormVO();

        BeanUtils.copyProperties(byId, signingRecordFormVO);

        List<SignedContentVO> signedContentVOList = signedContentService.getSignedContentVOList(signingRecordFormId);

        signingRecordFormVO.setSignedContentVOList(signedContentVOList);

        return signingRecordFormVO;
    }

    @Transactional
    @Override//增加签收明细
    public Boolean addFormById(List<SigningRecordFormDTO> signingRecordFormDTOList, DLPUser dlpUser) {

        boolean flag = true;

        int i = 0;

        for (SigningRecordFormDTO signingRecordFormDTO : signingRecordFormDTOList) {

            SignedBatchList signedBatchList = new SignedBatchList();

            BeanUtils.copyProperties(signingRecordFormDTO, signedBatchList);

            signedBatchList.setSignedBatchListId(IdWorker.get32UUID().toUpperCase());

            signedBatchList.setSignatoryId(dlpUser.getId());

            LambdaQueryWrapper<SignedBatchList> signedBatchListLambdaQueryWrapper = new LambdaQueryWrapper<>();

            signedBatchListLambdaQueryWrapper.eq(SignedBatchList::getSignedContentId, signedBatchList.getSignedContentId());
            //通过判断签收内容表里的签收批次表数量，进行批次字段的赋值
            signedBatchList.setBatch(signedBatchListService.list(signedBatchListLambdaQueryWrapper).size() + 1);

            SignedContent byId = signedContentService.getById(signedBatchList.getSignedContentId());

            byId.setSignQuantity(byId.getSignQuantity() + signedBatchList.getQuantity());

            if (byId.getSignQuantity() > byId.getTotalQuantity()) {
                throw new RuntimeException(String.format("签收数量不能大于采购数量"));
            }
            //遍历签收内容，判断试剂耗材是否签收完毕
            if (byId.getSignQuantity() == byId.getTotalQuantity()) {

                i = i + 1;
            }
            //创建入库登记任务
            StorageRegistrationForm storageRegistrationForm = new StorageRegistrationForm();

            storageRegistrationForm.setStorageRegistrationFormId(IdWorker.get32UUID().toUpperCase());

            storageRegistrationForm.setReagentConsumableId(byId.getReagentConsumableId());

            storageRegistrationForm.setQuantity(signedBatchList.getQuantity());

            if (signedContentService.updateById(byId) & signedBatchListService.save(signedBatchList)&storageRegistrationFormService.save(storageRegistrationForm)) {

                 flag = true;
            } else {
                 flag = false;
            }

        }
        LambdaQueryWrapper<SignedContent> signedContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

        String signingRecordFormId = signedContentService.getById(signingRecordFormDTOList.get(0).getSignedContentId()).getSigningRecordFormId();

        signedContentLambdaQueryWrapper.eq(SignedContent::getSigningRecordFormId, signingRecordFormId);

        List<SignedContent> list = signedContentService.list(signedContentLambdaQueryWrapper);

        SigningRecordForm byId = signingRecordFormService.getById(signingRecordFormId);
        //判断是否签收完全
        if (i == list.size()) {
            byId.setStatus(2);
        } else {
            byId.setStatus(1);
        }
        signingRecordFormService.updateById(byId);

        return flag;
    }

    @Override
    public SignedBatchList editFormById(SigningRecordFormDTO signingRecordFormDTO) {

        int i = 0;

        SignedBatchList byId = signedBatchListService.getById(signingRecordFormDTO.getSignedBatchListId());

        SignedContent signedContent = signedContentService.getById(signingRecordFormDTO.getSignedContentId());

        signedContent.setSignQuantity(signedContent.getSignQuantity() - byId.getQuantity() + signingRecordFormDTO.getQuantity());

        BeanUtils.copyProperties(signingRecordFormDTO, byId);

        LambdaQueryWrapper<SignedContent> signedContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

        String signingRecordFormId = signedContent.getSigningRecordFormId();

        signedContentLambdaQueryWrapper.eq(SignedContent::getSigningRecordFormId, signingRecordFormId);

        List<SignedContent> list = signedContentService.list(signedContentLambdaQueryWrapper);

        SigningRecordForm byId1 = signingRecordFormService.getById(signingRecordFormId);

        if (signedContent.getSignQuantity() > signedContent.getTotalQuantity()) {
            throw new RuntimeException(String.format("签收数量不能大于采购数量"));
        }
        //修改入库登记任务，试剂耗材入库数量
        LambdaQueryWrapper<StorageRegistrationForm> storageRegistrationFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        storageRegistrationFormLambdaQueryWrapper.eq(StorageRegistrationForm::getSignedBatchListId,byId.getSignedBatchListId());

        StorageRegistrationForm byId2 = storageRegistrationFormService.getById(storageRegistrationFormLambdaQueryWrapper);

        byId2.setQuantity(byId.getQuantity());

        if (signedContentService.updateById(signedContent) & signedBatchListService.updateById(byId)&storageRegistrationFormService.updateById(byId2)) {
        //遍历签收内容，判断每个试剂耗材是否签收完毕
            for (SignedContent content : list) {

                if (content.getTotalQuantity() == content.getSignQuantity()) {
                    i = i + 1;
                }
            }
            //判断是否签收完全
            if (i == list.size()) {
                byId1.setStatus(2);
                signingRecordFormService.updateById(byId1);
            }
            return byId;
        } else {
            return null;
        }
    }


}
