package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.StorageRegistrationFormDTO;
import digital.laboratory.platform.reagent.entity.CabinetForm;
import digital.laboratory.platform.reagent.entity.LatticeForm;
import digital.laboratory.platform.reagent.entity.StorageRegistrationForm;
import digital.laboratory.platform.reagent.entity.StorageRoomForm;
import digital.laboratory.platform.reagent.mapper.StorageRegistrationFormMapper;
import digital.laboratory.platform.reagent.service.CabinetFormService;
import digital.laboratory.platform.reagent.service.LatticeFormService;
import digital.laboratory.platform.reagent.service.StorageRegistrationFormService;
import digital.laboratory.platform.reagent.service.StorageRoomFormService;
import digital.laboratory.platform.reagent.vo.StorageRegistrationFormVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (试剂耗材入库登记表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材入库登记表) 服务实现类
 */
@Service
public class StorageRegistrationFormServiceImpl extends ServiceImpl<StorageRegistrationFormMapper, StorageRegistrationForm> implements StorageRegistrationFormService {

    @Autowired
    private StorageRegistrationFormService storageRegistrationFormService;

    @Autowired
    private LatticeFormService latticeFormService;

    @Autowired
    private CabinetFormService cabinetFormService;

    @Autowired
    private StorageRoomFormService storageRoomFormService;

    @Override//分页查看
    public List<StorageRegistrationFormVO> getStorageRegistrationFormVOList(QueryWrapper<StorageRegistrationForm> qw) {

        List<StorageRegistrationFormVO> storageRegistrationFormVOList = baseMapper.getStorageRegistrationFormVOList(qw);

        for (StorageRegistrationFormVO storageRegistrationFormVO : storageRegistrationFormVOList) {

            LatticeForm byId = latticeFormService.getById(storageRegistrationFormVO.getLatticeId());

            CabinetForm byId1 = cabinetFormService.getById(byId.getCabinetFormId());

            StorageRoomForm byId2 = storageRoomFormService.getById(byId1.getStorageRoomFormId());

            storageRegistrationFormVO.setLocation(byId2.getName() + byId1.getName() + byId.getNumber());
        }
        return storageRegistrationFormVOList;
    }


    @Override//增加入库登记表
    public List<StorageRegistrationForm> addFormById(List<StorageRegistrationFormDTO> storageRegistrationFormDTOList, DLPUser dlpUser) {

        List<StorageRegistrationForm> storageRegistrationForms = new ArrayList<>();

        for (StorageRegistrationFormDTO storageRegistrationFormDTO : storageRegistrationFormDTOList) {

            StorageRegistrationForm byId = storageRegistrationFormService.getById(storageRegistrationFormDTO.getStorageRegistrationFormId());

            BeanUtils.copyProperties(storageRegistrationFormDTO, byId);

            byId.setDepositorId(dlpUser.getId());

            byId.setDateOfArrival(LocalDateTime.now());

            storageRegistrationForms.add(byId);

        }
        if (storageRegistrationFormService.updateBatchById(storageRegistrationForms)) {
            return storageRegistrationForms;
        } else {
            return null;
        }
    }
}
